unit FieldEditUnit;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  CSVOps;

type

  { TFieldEditBox }

  TFieldEditBox = class(TForm)
    btAdd: TButton;
    btEdit: TButton;
    btRemove: TButton;
    btMoveUp: TButton;
    btMoveDown: TButton;
    btClose: TButton;
    edField: TEdit;
    lbFields: TListBox;
    procedure btAddClick(Sender: TObject);
    procedure btCloseClick(Sender: TObject);
    procedure btEditClick(Sender: TObject);
    procedure btMoveDownClick(Sender: TObject);
    procedure btMoveUpClick(Sender: TObject);
    procedure btRemoveClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormShow(Sender: TObject);
    procedure lbFieldsSelectionChange(Sender: TObject; User: boolean);
    procedure UpdateButtons(Sender: TObject);
  private
    Table: TCSVTable;
    procedure UpdateFields;
    procedure UpdateDefaultButton;
    procedure UpdateButtonState;
  public
    procedure Run(ATable: TCSVTable);
  end; 

var
  FieldEditBox: TFieldEditBox;

implementation

{$R *.lfm}

{ TFieldEditBox }

procedure TFieldEditBox.btCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TFieldEditBox.btEditClick(Sender: TObject);
begin
  Table.CellString[lbFields.ItemIndex, 0]:=edField.Text;
  lbFields.Items[lbFields.ItemIndex]:=edField.Text;
  edField.SetFocus;
end;

procedure TFieldEditBox.btMoveDownClick(Sender: TObject);
var
  Tmp: string;
begin
  Table.SwapFields(lbFields.ItemIndex, lbFields.ItemIndex + 1);
  Tmp:=lbFields.Items[lbFields.ItemIndex];
  lbFields.Items[lbFields.ItemIndex]:=lbFields.Items[lbFields.ItemIndex + 1];
  lbFields.Items[lbFields.ItemIndex + 1]:=Tmp;
  lbFields.ItemIndex:=lbFields.ItemIndex + 1;
  UpdateButtons(Sender);
end;

procedure TFieldEditBox.btMoveUpClick(Sender: TObject);
var
  Tmp: string;
begin
  Table.SwapFields(lbFields.ItemIndex, lbFields.ItemIndex - 1);
  Tmp:=lbFields.Items[lbFields.ItemIndex];
  lbFields.Items[lbFields.ItemIndex]:=lbFields.Items[lbFields.ItemIndex - 1];
  lbFields.Items[lbFields.ItemIndex - 1]:=Tmp;
  lbFields.ItemIndex:=lbFields.ItemIndex - 1;
  UpdateButtons(Sender);
end;

procedure TFieldEditBox.btRemoveClick(Sender: TObject);
begin
  Table.RemoveField(lbFields.ItemIndex);
  lbFields.Items.Delete(lbFields.ItemIndex);
  UpdateButtons(Sender);
end;

procedure TFieldEditBox.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key=27 then Close;
end;

procedure TFieldEditBox.FormShow(Sender: TObject);
begin
  UpdateFields;
  UpdateButtons(Sender);
end;

procedure TFieldEditBox.lbFieldsSelectionChange(Sender: TObject; User: boolean);
begin
  UpdateButtonState;
  if User then
    if lbFields.ItemIndex=-1 then
      edField.Text:=''
    else
      edField.Text:=lbFields.Items[lbFields.ItemIndex];
end;

procedure TFieldEditBox.UpdateFields;
var
  i: Integer;
begin
  lbFields.Clear;
  if Table.Width=0 then Exit;
  for i:=0 to Table.Width - 1 do lbFields.Items.Add(Table.CellString[i, 0]);
end;

procedure TFieldEditBox.UpdateButtons(Sender: TObject);
begin
  UpdateDefaultButton;
  UpdateButtonState;
end;

procedure TFieldEditBox.UpdateDefaultButton;
begin
  if edField.Focused then begin
    btClose.Default:=False;
    if lbFields.ItemIndex=-1 then begin
      btEdit.Default:=False;
      btAdd.Default:=True;
    end else begin
      btAdd.Default:=False;
      btEdit.Default:=True;
    end;
  end else begin
    btAdd.Default:=False;
    btEdit.Default:=False;
    btClose.Default:=True;
  end;
end;

procedure TFieldEditBox.UpdateButtonState;
var
  AnySelected: Boolean;
begin
  AnySelected:=lbFields.ItemIndex <> -1;
  btEdit.Enabled:=AnySelected;
  btRemove.Enabled:=AnySelected;
  btMoveUp.Enabled:=AnySelected and (lbFields.ItemIndex > 0);
  btMoveDown.Enabled:=AnySelected and (lbFields.ItemIndex < lbFields.Items.Count - 1);
end;

procedure TFieldEditBox.btAddClick(Sender: TObject);
var
  Field: string;
begin
  Field:=Trim(edField.Text);
  if Field='' then Field:=Trim(InputBox('Add Field', 'Field:', ''));
  if Field='' then begin
    edField.SetFocus;
    Exit;
  end;
  lbFields.Items.Add(Field);
  Table.AddField(Field);
  edField.Text:='';
  lbFields.ItemIndex:=-1;
  UpdateButtonState;
  edField.SetFocus;
end;

procedure TFieldEditBox.Run(ATable: TCSVTable);
begin
  Table:=ATable;
  ShowModal;
end;

end.

