unit MainUnit;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, Menus, Grids,
  CSVOps, LazHelp;

type

  { TMain }

  TMain = class(TForm)
    ApplicationProperties1: TApplicationProperties;
    LazHelp1: TLazHelp;
    LazHelpWindowedViewer1: TLazHelpWindowedViewer;
    MainMenu1: TMainMenu;
    mEdit: TMenuItem;
    mEditFields: TMenuItem;
    mEditBar1: TMenuItem;
    mEditInsertRow: TMenuItem;
    mEditDeleteRow: TMenuItem;
    mEditMoveUp: TMenuItem;
    mEditMoveDown: TMenuItem;
    mEditBar2: TMenuItem;
    mEditExtendedEditor: TMenuItem;
    mFileSaveACopy: TMenuItem;
    mHelpAbout: TMenuItem;
    mHelpBar1: TMenuItem;
    mHelpContents: TMenuItem;
    mHelp: TMenuItem;
    mFileExit: TMenuItem;
    mFileBar1: TMenuItem;
    mFileSaveAs: TMenuItem;
    mFileSave: TMenuItem;
    mFileOpen: TMenuItem;
    mFileNew: TMenuItem;
    mFile: TMenuItem;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    sgTable: TStringGrid;
    StringListLazHelpProvider1: TStringListLazHelpProvider;
    mHelpWebsite: TMenuItem;
    procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure mEditDeleteRowClick(Sender: TObject);
    procedure mEditExtendedEditorClick(Sender: TObject);
    procedure mEditFieldsClick(Sender: TObject);
    procedure mEditInsertRowClick(Sender: TObject);
    procedure mEditMoveDownClick(Sender: TObject);
    procedure mEditMoveUpClick(Sender: TObject);
    procedure mFileExitClick(Sender: TObject);
    procedure mFileNewClick(Sender: TObject);
    procedure mFileOpenClick(Sender: TObject);
    procedure mFileSaveACopyClick(Sender: TObject);
    procedure mFileSaveAsClick(Sender: TObject);
    procedure mFileSaveClick(Sender: TObject);
    procedure mHelpAboutClick(Sender: TObject);
    procedure mHelpContentsClick(Sender: TObject);
    procedure sgTableColRowMoved(Sender: TObject; IsColumn: Boolean; sIndex,
      tIndex: Integer);
    procedure sgTableDblClick(Sender: TObject);
    procedure sgTableEditingDone(Sender: TObject);
    procedure sgTableHeaderClick(Sender: TObject; IsColumn: Boolean;
      Index: Integer);
    procedure sgTableSetEditText(Sender: TObject; ACol, ARow: Integer;
      const Value: string);
    procedure mHelpWebsiteClick(Sender: TObject);
  private
    FFileName: string;
    FTable: TCSVTable;
    function GetModified: Boolean; inline;
    procedure SetFileName(const AValue: string);
    procedure UpdateGrid;
    function LoadTableFromFile(AFileName: string): Boolean;
    function SaveTableToFile(AFileName: string): Boolean;
  public
    property Modified: Boolean read GetModified;
    property FileName: string read FFileName write SetFileName;
    property Table: TCSVTable read FTable;
  end; 

var
  Main: TMain;

implementation

uses
  FieldEditUnit, EditorUnit, LCLIntf;

{$R *.lfm}

{ TMain }

procedure TMain.FormCreate(Sender: TObject);
begin
  FTable:=TCSVTable.Create;
  UpdateGrid;
end;

procedure TMain.FormCloseQuery(Sender: TObject; var CanClose: boolean);
begin
  CanClose:=True;
  if Modified and (MessageDlg('The table has been modified', 'Do you want to exit and lose the current changes?', mtConfirmation, mbYesNo, 0)=mrNo) then CanClose:=False;
end;

procedure TMain.FormDestroy(Sender: TObject);
begin
  FreeAndNil(FTable);
end;

procedure TMain.mEditDeleteRowClick(Sender: TObject);
var
  Index: Integer;
begin
  if not Assigned(Table) then Exit;
  Index:=sgTable.Row;
  if (Index < 1) or (Index >= Table.Height) then
    ShowMessage('Select a row first')
  else
    Table.DeleteRow(Index);
  UpdateGrid;
end;

procedure TMain.mEditExtendedEditorClick(Sender: TObject);
var
  Value: string;
begin
  if Assigned(Table) and (Table.Width > 0) and (Table.Height > 0) and (sgTable.Col > 0) and (sgTable.Row > 0) then begin
    if sgTable.Row=Table.Height then begin
      if Assigned(sgTable.Editor) then Value:=sgTable.Editor.Caption else Value:='';
    end else begin
      Value:=Table.CellString[sgTable.Col - 1, sgTable.Row];
    end;
    Value:=RunEditor('Edit ' + Table.CellString[sgTable.Col - 1, 0] + ' at row ' + IntToStr(sgTable.Row), Value);
    if sgTable.Row=Table.Height then begin
      if Trim(Value)='' then Exit;
      Table.InsertRow(Table.Height);
    end;
    Table.CellString[sgTable.Col - 1, sgTable.Row]:=Value;
    UpdateGrid;
  end;
end;

procedure TMain.mEditFieldsClick(Sender: TObject);
begin
  sgTable.EditorMode:=False;
  Application.CreateForm(TFieldEditBox, FieldEditBox);
  FieldEditBox.Run(Table);
  FreeAndNil(FieldEditBox);
  UpdateGrid;
end;

procedure TMain.mEditInsertRowClick(Sender: TObject);
var
  Index: Integer;
begin
  if not Assigned(Table) then Exit;
  if Table.Height=0 then begin
    mEditFieldsClick(Sender);
    Exit;
  end;
  if sgTable.Row < 1 then Index:=Table.Height else Index:=sgTable.Row;
  Table.InsertRow(Index);
  UpdateGrid;
end;

procedure TMain.mEditMoveDownClick(Sender: TObject);
begin
  if not Assigned(Table) then Exit;
  if sgTable.Row < 1 then Exit;
  if sgTable.Row >= Table.Height - 1then begin
    ShowMessage('The current row is already at the last position');
    Exit;
  end;
  Table.SwapRows(sgTable.Row, sgTable.Row + 1);
  UpdateGrid;
  sgTable.Row:=sgTable.Row + 1;
end;

procedure TMain.mEditMoveUpClick(Sender: TObject);
begin
  if not Assigned(Table) then Exit;
  if (sgTable.Row < 2) or (sgTable.Row >= Table.Height) then begin
    ShowMessage('The current row is already at the first position');
    Exit;
  end;
  Table.SwapRows(sgTable.Row, sgTable.Row - 1);
  UpdateGrid;
  sgTable.Row:=sgTable.Row - 1;
end;

procedure TMain.mFileExitClick(Sender: TObject);
begin
  Close;
end;

procedure TMain.mFileNewClick(Sender: TObject);
begin
  if Modified and (MessageDlg('The table has been modified', 'Do you want to proceed creating a new table and lose the current changes?', mtConfirmation, mbYesNo, 0)=mrNo) then Exit;
  sgTable.EditorMode:=False;
  FreeAndNil(FTable);
  FTable:=TCSVTable.Create;
  UpdateGrid;
  FileName:='';
end;

procedure TMain.mFileOpenClick(Sender: TObject);
begin
  if Modified and (MessageDlg('The table has been modified', 'Do you want to proceed opening another table and lose the current changes?', mtConfirmation, mbYesNo, 0)=mrNo) then Exit;
  OpenDialog1.FileName:=FileName;
  if not OpenDialog1.Execute then Exit;
  if LoadTableFromFile(OpenDialog1.FileName) then FileName:=OpenDialog1.FileName;
  UpdateGrid;
end;

procedure TMain.mFileSaveACopyClick(Sender: TObject);
begin
  SaveDialog1.FileName:=FileName;
  SaveDialog1.Title:='Save a Copy';
  if not SaveDialog1.Execute then Exit;
  SaveTableToFile(SaveDialog1.FileName);
end;

procedure TMain.mFileSaveAsClick(Sender: TObject);
begin
  SaveDialog1.FileName:=FileName;
  SaveDialog1.Title:='Save As';
  if not SaveDialog1.Execute then Exit;
  if SaveTableToFile(SaveDialog1.FileName) then begin
    FileName:=SaveDialog1.FileName;
    Table.Modified:=False;
  end;
end;

procedure TMain.mFileSaveClick(Sender: TObject);
begin
  if FileName='' then begin
    mFileSaveAsClick(Sender);
    Exit;
  end;
  if SaveTableToFile(FileName) then Table.Modified:=False;
end;

procedure TMain.mHelpAboutClick(Sender: TObject);
begin
  MessageDlg('About',
      'Runtime Table version 0.2' + LineEnding +
      'Copyright (C) 2012-2021 Kostas Michalopoulos' + LineEnding +
      '' + LineEnding +
      'This software is provided ''as-is'', without any express or implied' + LineEnding +
      'warranty.  In no event will the authors be held liable for any damages' + LineEnding +
      'arising from the use of this software.' + LineEnding +
      '' + LineEnding +
      'Permission is granted to anyone to use this software for any purpose,' + LineEnding +
      'including commercial applications, and to alter it and redistribute it' + LineEnding +
      'freely, subject to the following restrictions:' + LineEnding +
      '' + LineEnding +
      '1. The origin of this software must not be misrepresented; you must not' + LineEnding +
      '   claim that you wrote the original software. If you use this software' + LineEnding +
      '   in a product, an acknowledgment in the product documentation would be' + LineEnding +
      '   appreciated but is not required.' + LineEnding +
      '2. Altered source versions must be plainly marked as such, and must not be' + LineEnding +
      '   misrepresented as being the original software.' + LineEnding +
      '3. This notice may not be removed or altered from any source distribution.' + LineEnding +
      '' + LineEnding +
      'Kostas Michalopoulos <badsector@runtimeterror.com>' + LineEnding + LineEnding +
      'Runtime Table was made using Free Pascal and Lazarus.' + LineEnding +
      'Find the latest version at http://runtimeterror.com/tools/table',
    mtInformation, [mbOK], 0);
end;

procedure TMain.mHelpContentsClick(Sender: TObject);
begin
  LazHelpWindowedViewer1.ShowHelp;
end;

procedure TMain.sgTableColRowMoved(Sender: TObject; IsColumn: Boolean; sIndex,
  tIndex: Integer);
begin
  if (sIndex=0) or (tIndex=0) then Exit;
  if IsColumn then begin
    Table.SwapFields(sIndex - 1, tIndex - 1);
  end else if (sIndex < Table.Height) and (tIndex < Table.Height) then begin
    Table.SwapRows(sIndex, tIndex);
  end;
  sgTable.EditorMode:=False;
  UpdateGrid;
end;

procedure TMain.sgTableDblClick(Sender: TObject);
begin
  mEditFieldsClick(Sender);
end;

procedure TMain.sgTableEditingDone(Sender: TObject);
begin
  UpdateGrid;
end;

procedure TMain.sgTableHeaderClick(Sender: TObject; IsColumn: Boolean;
  Index: Integer);
begin
  if (not IsColumn) or (Index < 1) then Exit;
  Table.CellString[Index - 1, 0]:=InputBox('Rename Field', 'New Name', Table.CellString[Index - 1, 0]);
  UpdateGrid;
end;

procedure TMain.sgTableSetEditText(Sender: TObject; ACol, ARow: Integer;
  const Value: string);
begin
  if not Assigned(Table) then Exit;
  if ARow=Table.Height then begin
    if Trim(Value)='' then Exit;
    Table.InsertRow(Table.Height);
  end;
  Table.CellString[ACol - 1, ARow]:=Value;
end;

procedure TMain.mHelpWebsiteClick(Sender: TObject);
begin
  OpenURL('http://runtimeterror.com/tools/table');
end;

procedure TMain.UpdateGrid;
var
  y: Integer;
  x: Integer;
begin
  if not Assigned(Table) then Exit;
  if (Table.Width=0) and (Table.Height=0) then begin
    sgTable.Clean;
    sgTable.RowCount:=1;
    sgTable.ColCount:=1;
    Exit;
  end;
  sgTable.RowCount:=Table.Height + 1;
  sgTable.ColCount:=Table.Width + 1;
  for y:=0 to Table.Height - 1 do begin
    if y > 0 then sgTable.Cells[0, y]:=IntToStr(y);
    for x:=0 to Table.Width - 1 do
      sgTable.Cells[x + 1, y]:=Table.CellString[x, y];
  end;
  for x:=1 to Table.Width do sgTable.Cells[x, Table.Height]:='';
  sgTable.Cells[0, Table.Height]:='+';
  sgTable.ColWidths[0]:=40;
end;

function TMain.LoadTableFromFile(AFileName: string): Boolean;
begin
  Result:=False;
  try
    Table.LoadFromFile(AFileName);
    Result:=True;
  except
    MessageDlg('Failed to load', 'The file ' + AFileName + ' could not be loaded.', mtError, [mbOK], 0);
  end;
end;

function TMain.SaveTableToFile(AFileName: string): Boolean;
begin
  Result:=False;
  try
    Table.SaveToFile(AFileName);
    Result:=True;
  except
    MessageDlg('Failed to save', 'The file ' + AFileName + ' could not be saved.', mtError, [mbOK], 0);
  end;
end;

procedure TMain.SetFileName(const AValue: string);
begin
  if FFileName=AValue then Exit;
  FFileName:=AValue;
  if FFileName='' then Caption:='Runtime Table' else Caption:=ExtractFileName(FFileName) + ' - Runtime Table';
  ApplicationProperties1.Title:=Caption;
end;

function TMain.GetModified: Boolean;
begin
  Result:=Table.Modified;
end;

end.

